#ifndef FACE_HH
#define FACE_HH

/*
  James R. Diebel
  Stanford University
  
  Started: 27 August 2004
  Last revised: 27 Sugust 2004

  face.hh - class definition for a face in a 3d surface mesh as defined
  in the header file mesh.hh.

  Depends on:
  - Vec3d class (vec3d.hh)
  - Mesh class (mesh.hh)
  - Vert class (vert.hh)
  - Edge class (edge.hh)

  Used almost exclusively by Mesh class (mesh.hh)
*/

#include <iostream>
#include "vec3d.hh"
#include "mesh.hh"
#include "vert.hh"
#include "edge.hh"

using namespace std;

namespace bmtk {

class Face {
public:
  // Data
  int nf,ne; // number of neighboring faces, defined edges
  int i; // index of self within global face list
  int flag; // flag used in flood-fill operations
  Vec3d n; // normal vector of face
  Vec3d* ns; // array of the (m->nd) different normal vectors
  Vec3d n0; // original normal vector of face
  Vec3d nr; // reference vector, used temporarily in some routines
  Vec3d x; // geometric center of triangular face
  bool bound; // whether the face is on the mesh boundary
  float d; // projected distace to the origin along normal vector
  float dr; // projected distance to origin along region normal vector
  float s; // area of face
  class Mesh *m; // link to parent mesh
  class Vert *v[3]; // list of vertices of face
  class Edge *e[3]; // list of edges of face
  class Face *f[3]; // list of neighboring faces
  class Region *r; // pointer to current face region

  // Constructor/Destructor
  Face();
  ~Face();

  // Member functions
  void findNormal();
  void findCenter();
  void findDistance();
  void findArea();
  void blur();
  int indexOf(Vert& vp);
};

// Membership tests
bool operator < (Vert &vp, Face &fp);
bool operator < (Edge &ep, Face &fp);

// Define << operator to handle output to screen
ostream& operator << (ostream& os, const Face &f);

} // namespace bmtk

#endif
