#ifndef REGION_HH
#define REGION_HH

/*
  James R. Diebel
  Stanford University
  
  Started: 27 August 2004
  Last revised: 27 Sugust 2004

  region.hh - class definition for a face region class in a 3d surface mesh
  as defined in the header file mesh.hh

  Depends on:
  - Vec3d class (vec3d.hh)
  - Mesh class (mesh.hh)
  - Face class (face.hh)

  Used almost exclusively by Mesh class (mesh.hh)
*/

#include <iostream>
#include <vector>
#include "vec3d.hh"
#include "mesh.hh"

namespace bmtk {

using namespace std;

class Region {
public:
  // Data
  vector<int> vi; // list of vertex indices defining approx. region boundary
  vector<int> tvi; // triangulated list of the same
  vector<Vec3d> ni; // the fit normal vector for each vertex
  vector<int> fi; // list of faces in region
  int i; // self-reference in global region list
  int p; // population of region
  Vec3d n; // normal vector
  Vec3d* ns; // mean normals for each of the (m->nd) data sets
  float ssn2; // sum of normal vector variance (sum sigma_n squared)
  class Mesh *m; // pointer to parent mesh

  // Constructor/Destructor
  Region();
  ~Region();
};

} // namespace bmtk

#endif
