/*
  James R. Diebel
  Stanford University

  Started: 27 August 2004
  Last revised: 27 Sugust 2004

  edge.cc - class implementation of Edge class (edge.hh)

  Depends on edge.hh
*/

#include "bmtk/edge.hh"
#include <math.h>

namespace bmtk {

// Constructor/Destructor
Edge::Edge() {
  flag = nf = 0; i = -1;
  a = l = psi = 0.0;
}
Edge::~Edge() {
}

// Member functions
void Edge::findAngle() {
  if (nf == 2) a = acos(f[0]->n.dot(f[1]->n));
}

void Edge::findPotential() {
  if (nf == 2) {
    l = v[0]->x.distTo(v[1]->x);
    psi = m->psiSpline->evalf(1-f[0]->n.dot(f[1]->n));
  }
}

// finds gradient of psi WRT position of vertex i
Vec3d Edge::gradient(Vert& vi) {
  Vec3d gradPsi;

  if (nf == 2) {
    // Compute the gradient of potential WRT
    float dpsi_dy = m->psiSpline->evaldf(1-f[1]->n.dot(f[0]->n));

    int ivi0 = f[0]->indexOf(vi);
    if (ivi0 != -1) {
      Vec3d n0pn1 = f[0]->n - f[0]->n.dot(f[1]->n)*f[1]->n;
      gradPsi += dpsi_dy*n0pn1.cross(f[0]->v[(ivi0 + 2)%3]->x -
             f[0]->v[(ivi0 + 1)%3]->x);
    }

    int ivi1 = f[1]->indexOf(vi);
    if (ivi1 != -1) {
      Vec3d n1pn0 = f[1]->n - f[1]->n.dot(f[0]->n)*f[0]->n;
      gradPsi += dpsi_dy*n1pn0.cross(f[1]->v[(ivi1 + 2)%3]->x -
             f[1]->v[(ivi1 + 1)%3]->x);
    }
  }

  return gradPsi;
}

void Edge::findLength() {
  l = v[0]->x.distTo(v[1]->x);
}

int Edge::indexOf(Vert& vp) {
  if (v[0] == &vp) return 0;
  if (v[1] == &vp) return 1;
  return -1;
}

// Equality tests
bool Edge::operator == (const Edge &e) const {
  return ((v[0] == e.v[0] && v[1] == e.v[1]) ||
    (v[1] == e.v[0] && v[0] == e.v[1]));
}
bool Edge::operator != (const Edge &e) const {
  return ((v[0] != e.v[0] || v[1] != e.v[1]) &&
    (v[1] != e.v[0] || v[0] != e.v[1]));
}

// Membership tests
bool operator < (Vert &vp, Edge &ep) {
  return (ep.v[0] == &vp || ep.v[1] == &vp);
}

// Define << operator to handle output to screen
ostream& operator << (ostream& os, const Edge &e) {
  return os<<"e"<<e.i<<": "<<e.v[0]->i<<", "<<e.v[1]->i<<endl<<flush;
}

} // namespace bmtk
