/*
  James R. Diebel
  Stanford University

  Started: 24 August 2004
  Last revised:

  mesh_vert.cc - class implementation of Mesh class (mesh.hh)

  Depends on:
  - Mesh class (mesh.hh)
*/

#include "bmtk/mesh.hh"
#include <iostream>

namespace bmtk {

////////////////////////
// Per-Face Functions //
////////////////////////
void Mesh::findFaceNormals() {
  if (ut) tFN->start();
  if (po) cout << "- Computing face normals..." << flush;
  for (int i=0;i<nf;i++) f[i].findNormal();
  if (po) cout << "Done. " << flush;
  if (po) tFN->printMark();
  if (ut) tFN->mark();
}

void Mesh::findFaceCenters() {
  if (po) cout << "- Computing face centers..." << flush;
  for (int i=0;i<nf;i++) f[i].findCenter();
  if (po) cout << "Done." << endl << flush;
}

void Mesh::findFaceDistances() {
  if (po) cout << "- Computing face projected distances..." << flush;
  for (int i=0;i<nf;i++) f[i].findDistance();
  if (po) cout << "Done." << endl << flush;
}

void Mesh::findFaceAreas() {
  if (po) cout << "- Computing face areas..." << flush;
  for (int i=0;i<nf;i++) f[i].findArea();
  if (po) cout << "Done." << endl << flush;
}

void Mesh::findFaceProps() {
  if (po) cout << "- Computing face properties..." << flush;
  for (int i=0;i<nf;i++) {
    f[i].findNormal();
    f[i].findCenter();
    f[i].findDistance();
    f[i].findArea();
  }
  if (po) cout << "Done." << endl << flush;
}

void Mesh::saveRefNormals() {
  if (po) cout << "- Saving reference face normals..." << flush;
  for (int i=0;i<nf;i++) f[i].nr = f[i].n;
  if (po) cout << "Done." << endl << flush;
}

void Mesh::blurFaceNormals(int num) {
  if (po) cout << "- Blurring face normals..." << flush;
  bool temp = po;
  po = false;
  for (int i=0;i<num;i++) {
    saveRefNormals();
    for (int i=0;i<nf;i++) f[i].blur();
  }
  po = temp;
  if (po) cout << "Done." << endl << flush;
}

void Mesh::resetFaceFlags() {
  if (po) cout << "- Reseting face flags..." << flush;
  for (int i=0;i<nf;i++) f[i].flag = -1;
  if (po) cout << "Done." << endl << flush;
}

} // namespace bmtk
