/*
  James R. Diebel
  Stanford University

  Started: 2 December 2004

  msh2ply.cc - Utility to convert a MSH file to a PLY file

  Depends on:
  - Mesh class (mesh.hh)
  -- Vec3d class (vec3d.hh)
  -- Vert class (vert.hh)
  -- Edge class (edge.hh)
  -- Face class (face.hh)
  - Timer class (timer.hh)
  - TriMesh class (trimesh.h)
*/


#include <iostream>
#include "bmtk/mesh.hh"
#include "bmtk/timer.hh"
#include <trimesh/trimesh.h>
#include <trimesh/trimesh_endian.h>

using namespace std;
using namespace bmtk;

int main (int argc, char *argv[]) {
  // if bad command line args, give usage instructions and exit
  if (argc < 3) {
    cout << endl
	 << "Usage: msh2ply infile.msh outfile.ply" << endl
	 << "    infile.msh must be an msh file" << endl
	 << "    outfile.ply is a little-endian binary ply file" << endl;
    exit(1);
  }

  // read Mesh from file
  Mesh m;
  m.readMesh(argv[1]);

  // Export Mesh to ply format
  m.exportToPLY(argv[2]);

  return 0;
}
