/*
  James R. Diebel
  Stanford University

  Started: 10 January 2005

  smoothmsh.cc - Utility to smooth a MSH file

  Depends on:
  - Mesh class (mesh.hh)
  -- Vec3d class (vec3d.hh)
  -- Vert class (vert.hh)
  -- Edge class (edge.hh)
  -- Face class (face.hh)
  - Timer class (timer.hh)
  - TriMesh class (trimesh.h)
*/


#include <iostream>
#include "bmtk/mesh.hh"
#include "bmtk/timer.hh"
#include <trimesh/trimesh.h>

using namespace std;
using namespace bmtk;

int main (int argc, char *argv[]) {
  // if bad command line args, give usage instructions and exit
  if (argc < 2) {
    cout << endl
	 << "Usage: smoothmsh infile.msh" << endl
	 << "    infile.msh must be an msh file" << endl;
    exit(1);
  }

  // read Mesh from file
  Mesh m;
  m.readMesh(argv[1]);

  // find mesh properties that are independent of options
  m.findMeshProps();
  //m.normalize();
  m.findRegionProps();

  // find mesh properties that depend on options
  m.readOptions("mrf_options.dat");
  m.findMeshPotential();
  m.findLocalEdgePotentials();
  m.findVertGradients();
  m.findSearchDirs(true);

  // Run the optimization
  m.runCG();

  // Save the mesh to file
  m.writeMesh(m.filename);

  return 0;
}
