/*
  James R. Diebel
  Stanford University

  Started: 27 August 2004
  Last revised: 27 Sugust 2004

  timer.cc - class implementation of Timer class (timer.hh)

  Depends on timer.hh
*/

#include "bmtk/timer.hh"

namespace bmtk {

Timer::Timer() {
  totalTime = storeTime = 0;
}

Timer::Timer(char* prefix_, char* suffix_) {
  prefix = prefix_;
  suffix = suffix_;
  totalTime = storeTime = 0;
}

float Timer::read() {
  return float(totalTime)/float(CLOCKS_PER_SEC);
}

float Timer::start() {
  startTime = clock();
  return read();
}

float Timer::stop() {
  stopTime = clock();
  totalTime += (stopTime - startTime);
  return read();
}

float Timer::glance() {
  stop();
  start();
  return read();
}

void Timer::reset() {
  totalTime = 0;
}

float Timer::peek() {
  return float(storeTime)/float(CLOCKS_PER_SEC);
}

float Timer::store() {
  storeTime += totalTime;
  return peek();
}

void Timer::resetStore() {
  storeTime = 0;
}

void Timer::resetAll() {
  reset();
  resetStore();
}

float Timer::recall() {
  totalTime = storeTime;
  return read();
}

float Timer::mark() {
  stop();
  float reading = read();
  store();
  reset();
  return reading;
}

float Timer::print() {
  cout << prefix << "Timer: " << read() << " seconds"
       << suffix << flush;
  return read();
}

float Timer::printGlance() {
  float reading = glance();
  cout << prefix << "Timer: " << reading << " seconds"
       << suffix << flush;
  return reading;
}

float Timer::printStore() {
  cout << prefix << "Total time: " << peek() << " seconds"
       << suffix << flush;
  return peek();
}

float Timer::printStore(char* label) {
  cout << prefix << label << ": " << peek() << " seconds"
       << suffix << flush;
  return peek();
}

float Timer::printMark() {
  stop();
  float reading = read();
  store();
  reset();
  cout << prefix << "(" << reading << "/"
       << peek() << ")" << suffix << flush;
  start();
  return reading;
}

float Timer::printMark(char* label) {
  stop();
  float reading = read();
  store();
  reset();
  cout << prefix << label << ": " << reading << " seconds of "
       << peek() << " seconds in total" << suffix << flush;
  start();
  return reading;
}

float Timer::printStop() {
  stop();
  float reading = read();
  store();
  reset();
  cout << prefix << reading << " of "
       << peek() << suffix << flush;
  return reading;
}

float Timer::printStop(char* label) {
  stop();
  float reading = read();
  store();
  reset();
  cout << prefix << label << ": " << reading << " seconds of "
       << peek() << " seconds in total" << suffix << flush;
  return reading;
}

} // namespace bmtk
